import matplotlib.pyplot as plt
import numpy as np

def yearly_totals(column_titles, data):
    '''
    Pulls out the each value contained in the 'Year' entry of the csv file, then counts the 
    occurence of each year. This represents the number of homicide entries for each year, so 
    by counting these entries the total number of homocides is counted.
    '''

    years = data[:, column_titles['Year']]                           
    unique_year, count = np.unique(years, return_counts=True) 
    unique_year = [int(x) for x in unique_year]                 
    count = [int(x) for x in count]

    return list(zip(unique_year, count))


def totals_barplot(fig, column_titles, data, yearly_data):
    ''' Create a bar plot totaling the number of homicides every year'''
    
    year, total = zip(*yearly_data)

    # Creates and titles a graph
    ax = fig.add_subplot(221)
    plt.title('Homicides per Year')
    
    # Format axis and ticks
    plt.ylabel('Number of Homicides')
    ax.yaxis.set_ticks_position('both')
    plt.tick_params(axis='y', direction='in')
    plt.xlabel('Year')

    # Determines the x-ticks based from the highest and lowest years 
    plt.xticks(range(int(min(year)), int(max(year))+1), rotation='vertical') 
    ax.bar(year, total, edgecolor='black')
    
def total_changes(fig, column_titles, data, changes):
    '''
    Plot the differences of the yearly number of homicides in Chicago.
    '''
    
    changes.sort() # sorts by year
    unique_year, total = zip(*changes)
    
    # Calculate differences
    differences = np.diff(np.array(total))
    
    # Creates and titles a graph
    ax = fig.add_subplot(222)
    plt.title('Recent Homicide Rate Change')
    plt.grid(True)
    
    # Format axis and ticks
    plt.ylabel('Homicide Change')
    plt.yticks(range((min(differences)//40)*40, (max(differences)//40)*40+40, 40))
    plt.tick_params(axis='y', direction='in')
    plt.xlabel('Year')
    plt.xticks(range(unique_year[1], max(unique_year)+1, 2))
    
   
    ax.plot(unique_year[1:], differences, 'black')  # plot line
    ax.plot(unique_year[1:], differences, 'bo')     # plot dots
    

def locations_pieplot(fig, column_titles, data):
    '''
    Draw a pie chart of the most common location descriptions.
    E.g. STREET, ALLEY etc.
    '''

    # Extract relevant data
    locations = data[:, column_titles['Location Description']]
    labels, counts = np.unique(locations, return_counts=True)
    counts = [int(x) for x in counts]
    locations = list(zip(counts, labels))
    locations.sort(reverse=True)

    # Group the smaller location descriptions together as OTHER
    other_locations = (sum(x for x, y in locations[6:]), 'OTHER')
    locations = locations[:6]
    locations.append(other_locations)
    counts, labels = zip(*locations) 

    # Generate and title the plot
    ax = fig.add_subplot(223)
    plt.title('Homicide Locations')
    ax.axis('equal')

    ax.pie(counts, labels=labels, autopct='%1.1f%%',  startangle=90, pctdistance=0.8)

def districts_scatter(fig, column_titles, data):
    '''
    Draw a scatter plot of the districts:
        The size of the circles represents the amount of homicides in a district.
        The x-axis are the districts
        The y-axis shows the ratio of how many homicides have resutled in an arrest.
    '''
    
    # Extract relevant data
    all_districts = data[:, column_titles['District']]
    district, total = np.unique(all_districts, return_counts=True)
    district = [int(x) for x in district]
    district_counts = list(zip(district, total))
    district_counts.sort()
    district, total = zip(*district_counts)

    # Create a dictionary for the districts to store the number of arrest made
    arrests = {}
    for dist in district:
            arrests[dist] = 0

    # Count the number of arrest made for each district
    for i in range(len(data)):
        if data[i][column_titles['Arrest']] == 'TRUE':
            arrests[int(data[i][column_titles['District']])] += 1

    # Calculate ratio of solved/unsolved cases 
    solved = []
    for i, key in enumerate(arrests.keys()):
        solved.append((arrests[key] / total[i]) *100)

    # Resize circles
    total = [4*x for x in total]

    # Generate and title plot
    ax = fig.add_subplot(121)
    plt.title('Homicide Arrests by District')
    
    # Format labels and axis
    plt.ylabel('Percent of Cases Resulting in an Arrest')
    plt.yticks(range(int(min(solved)/5)*5-5, int(max(solved)/5)*5+5, 5))
    ax.yaxis.set_ticks_position('both')
    plt.tick_params(axis='y', direction='in')
    plt.xlabel('District (Dot Size Inidcates Number of Total Recorded Homicides)')
    plt.xticks(range(0, max(district)+1, 1))

    # Plot scatter plot with random numbers
    colors = np.random.rand(len(district))
    ax.scatter(district, solved, s=total, alpha=0.8, c=colors)

def homicide_heatmap(fig, column_titles, data):
    '''
    Draws a heatmap of homicides between 2001 and 2016 over a map of Chicago
    '''
    
    # Get X/Y coordinates
    a = data[:, column_titles["X Coordinate"]]
    b = data[:, column_titles["Y Coordinate"]]
    x,y  = [], []
    for i in range(len(a)):
        # check if coordinates can be cast to int
        if a[i].isdigit() and b[i].isdigit():
            # some homicides have location (0,0). filter them out
            if int(a[i]) != 0 and int(b[i]) != 0:
                x.append(int(a[i]))
                y.append(int(b[i]))
            
    ax = fig.add_subplot(122)
    
    # Set limits
    plt.xlim(min(x)-5000, max(x)+5000)
    plt.ylim(min(y)-5000, max(y)+5000)
    
    # Load map of chicago
    img = plt.imread('chicago_map.PNG')
    # Draw map. extent: stretching and adjusting map to line up with coordinates
    plt.imshow(img, zorder=0, extent=[min(x)-93000, max(x)+90000,min(y)-45000, max(y)+60000]) 
    
    plt.title('Homicide Heatmap')
    
    # Heatmap:
    ax.hexbin(x, y, gridsize=60, bins='log', alpha=0.65, cmap='jet', mincnt=1)
    ax.tick_params(bottom='off', left='off',labelbottom='off', labelleft='off')

    
def readfile():
    '''
    Read and parse the dcsv data file.
    Returns a dictionary containting the column titles with their indices
    and a numpy array containing the data. 
    '''

    # open and read the csv file
    fin = open('chicago_data.csv')
    line = fin.readline()
    
    # first line contains column titles
    heads = line.rstrip().split(',')
    
    # create a dictionary with column titles and their indices
    column_titles = {}
    for i, head in enumerate(heads):
        column_titles[head] = i  

    # read data into list
    line = fin.readline()
    data = []
    while line:
        values = line.rstrip().split(',')
        data.append(values)
        line = fin.readline()

    fin.close()
    
    # create a numpy array from data
    return column_titles, np.array(data)
    
    
def main():  
    column_titles, data = readfile() 
    
    # First plot
    fig = plt.figure(figsize=(14, 7))

    yearly_data = yearly_totals(column_titles, data)

    totals_barplot(fig, column_titles, data, yearly_data)

    total_changes(fig, column_titles, data, yearly_data)

    locations_pieplot(fig, column_titles, data)
    
    plt.tight_layout()

    # Second plot
    fig = plt.figure(figsize=(14, 7))
    
    districts_scatter(fig, column_titles, data)
    homicide_heatmap(fig, column_titles, data)
    
    plt.tight_layout()
    plt.show()

if __name__ == "__main__":
    main()

